// created on 6/10/2005 at 3:15 PM
/*
 *   Copyright (c) 2005, Alexandros Frantzis (alf82 [at] freemail [dot] gr)
 *
 *   This file is part of Bless.
 *
 *   Bless is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   Bless is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Bless; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
using Glade;
using Gtk;
using System;
using Bless.Util;

namespace Bless.Gui {

///<summary>
/// An advanced statusbar for Bless
///</summary>
class Infobar : Gtk.HPaned
{
	Statusbar MessageStatusbar;
	Statusbar DummyStatusbar;
	Statusbar OffsetStatusbar;
	Statusbar SelectionStatusbar;
	Statusbar OverwriteStatusbar;
	Tooltips tips;
	
	DataBook dataBook;
	
	int numberBase;
	
	///<summary>
	///Get or set the visibility of the Offset statusbar
	///</summary>
	public bool OffsetVisible {
		get { return OffsetStatusbar.Visible; }
		set { OffsetStatusbar.Visible=value; AssignResizeGrip(); }
	}
	
	///<summary>
	///Get or set the visibility of the Selection statusbar
	///</summary>
	public bool SelectionVisible {
		get { return SelectionStatusbar.Visible; }
		set { SelectionStatusbar.Visible=value; AssignResizeGrip(); }
	}
	
	///<summary>
	///Get or set the visibility of the Overwrite statusbar
	///</summary>
	public bool OverwriteVisible {
		get { return OverwriteStatusbar.Visible; }
		set { OverwriteStatusbar.Visible=value; AssignResizeGrip(); }
	}
	
	public int NumberBase {
		set { 
			numberBase=value;
			
			if (dataBook.NPages>0) {
				DataView curdv=((DataViewDisplay)dataBook.CurrentPageWidget).View;
			
				UpdateOffset(curdv);
				UpdateSelection(curdv);
			}
		}
		
		get {
			return numberBase;
		}
	}
	
	public Infobar(DataBook db)
	{
		dataBook=db;
		
		// create bars
		MessageStatusbar=new Statusbar();
		
		OffsetStatusbar=new Statusbar();
		OffsetStatusbar.WidthRequest=270;
		
		SelectionStatusbar=new Statusbar();
		SelectionStatusbar.WidthRequest=270;
		
		OverwriteStatusbar=new Statusbar();
		OverwriteStatusbar.WidthRequest=60;
		
		DummyStatusbar=new Statusbar();
		DummyStatusbar.Visible=false;
		
		EventBox OffsetEB=new EventBox();
		OffsetEB.Add(OffsetStatusbar);
		OffsetEB.ButtonPressEvent+=ChangeNumberBase;
		
		EventBox SelectionEB=new EventBox();
		SelectionEB.Add(SelectionStatusbar);
		SelectionEB.ButtonPressEvent+=ChangeNumberBase;
		
		EventBox OverwriteEB=new EventBox();
		OverwriteEB.Add(OverwriteStatusbar);
		OverwriteEB.ButtonPressEvent+=OnOverwriteStatusbarPressed;
		
		// create hbox to put bars in
		HBox StatusHBox=new HBox();
		StatusHBox.PackStart(DummyStatusbar, false, true, 0);
		StatusHBox.PackStart(OffsetEB, false, true, 0);
		StatusHBox.PackStart(SelectionEB, false, true, 0);
		StatusHBox.PackStart(OverwriteEB, false, true, 0);
		
		// align the hbox
		Alignment StatusAlignment=new Alignment(1.0f, 0.5f, 0.0f, 1.0f);
		StatusAlignment.Add(StatusHBox);
		
		this.Pack1(MessageStatusbar, true, true);
		this.Pack2(StatusAlignment, false, true);
		
		this.NumberBase=16;
		
		this.MapEvent+=OnMapEvent;
		this.ShowAll();
	}

	void OnMapEvent(object o, EventArgs args)
	{
		AssignResizeGrip();
	}
	
	///<summary>Displays a message in the infobar (for 5 sec)</summary>
	public void DisplayMessage(string message)
	{
		MessageStatusbar.Pop(0);	
		MessageStatusbar.Push(0, message);
		GLib.Timeout.Add(5000, ClearStatusMessage);
	}

	///<summary>Clears the status bar message (callback)</summary>
	bool ClearStatusMessage()
	{
		MessageStatusbar.Pop(0);
		return false;
	}
	
	///<summary>
	/// Updates all the information in the infobar with data from the specified DataView,
	/// if the specified DataView is active
	///</summary>
	public void Update(DataView dv)
	{
		UpdateOffset(dv);
		UpdateSelection(dv);
		UpdateOverwrite(dv);
	}
	
	///<summary>
	/// Updates the cursor status bar with data from the specified DataView,
	/// if the specified DataView is active
	///</summary>
	public void UpdateOffset(DataView dv)
	{	
		if (dataBook.NPages == 0)
			return;
		
		DataView curdv=((DataViewDisplay)dataBook.CurrentPageWidget).View;
		if (curdv!=dv)
			return;
	
		long coffset=dv.CursorOffset;
		OffsetStatusbar.Pop(0);
		
		string coffsetString=BaseConverter.ConvertToString(coffset, numberBase, true, false); 
		string sizeString=BaseConverter.ConvertToString(dv.Buffer.Size-1, numberBase, true, false);
		
		string str=string.Format("Offset: {0} / {1}", coffsetString, sizeString);
		OffsetStatusbar.Push(0, str);
	}
	
	///<summary>
	/// Updates the selection status bar with data from the specified DataView,
	/// if the specified DataView is active
	///</summary>
	public void UpdateSelection(DataView dv)
	{
		if (dataBook.NPages == 0)
			return;

		DataView curdv=((DataViewDisplay)dataBook.CurrentPageWidget).View;
		if (curdv!=dv)
			return;
	
		Bless.Util.Range sel=dv.Selection;
		sel.Sort();
		SelectionStatusbar.Pop(0);
		string str;
	
		if (sel.IsEmpty()==true)
			str="Selection: None";
		else {
			string startString=BaseConverter.ConvertToString(sel.Start, numberBase, true, false);
			string endString=BaseConverter.ConvertToString(sel.End, numberBase, true, false);
			string sizeString=BaseConverter.ConvertToString(sel.Size, numberBase, true, false);
			
			str=string.Format("Selection: {0} to {1} ({2} bytes)", startString, endString, sizeString);
		}
		
		SelectionStatusbar.Push(0, str);
	}
	
	///<summary>
	/// Updates the overwrite status bar with data from the specified DataView,
	/// if the specified DataView is active
	///</summary>
	public void UpdateOverwrite(DataView dv)
	{	
		if (dataBook.NPages == 0)
			return;
		
		DataView curdv=((DataViewDisplay)dataBook.CurrentPageWidget).View;
		if (curdv!=dv)
			return;
	
		OverwriteStatusbar.Pop(0);
		if (dv.Overwrite==true)
			OverwriteStatusbar.Push(0, " OVR");
		else
			OverwriteStatusbar.Push(0, " INS");
	}
	
	///<summary>
	/// Clears the information displayed in the infobar
	///</summary>
	public void ClearInfo()
	{
		OffsetStatusbar.Pop(0);
		OffsetStatusbar.Push(0, "Offset: -");
			
		SelectionStatusbar.Pop(0);
		SelectionStatusbar.Push(0, "Selection: None");
	}
	
	///<summary>Handle button press on Ins/Ovr statusbar</summary>
	void OnOverwriteStatusbarPressed (object o, ButtonPressEventArgs args) 
	{
		if (dataBook.NPages > 0) {
			Gdk.EventButton e=args.Event;
			// ignore double and triple-clicks
			if (e.Type!=Gdk.EventType.ButtonPress)
				return;
			
			// set edit mode
			DataView dv=((DataViewDisplay)dataBook.CurrentPageWidget).View;
			dv.Overwrite=!dv.Overwrite;
		}
	}
	
	void ChangeNumberBase(object o, ButtonPressEventArgs args)
	{
		if (dataBook.NPages == 0)
			return;
			
		Gdk.EventButton e=args.Event;
		// ignore double and triple-clicks
		if (e.Type!=Gdk.EventType.ButtonPress)
			return;
		
		// cycle 8, 10 and 16 number bases
		if (this.NumberBase==8)
			this.NumberBase=10;
		else if (this.NumberBase==10)
			this.NumberBase=16;
		else if (this.NumberBase==16)
			this.NumberBase=8;
	}
	
	///<summary>Assign the resize grip to a visible statusbar</summary>
	void AssignResizeGrip()
	{
		// clear all grips
		OffsetStatusbar.HasResizeGrip=false;
		SelectionStatusbar.HasResizeGrip=false;
		MessageStatusbar.HasResizeGrip=false;
		OverwriteStatusbar.HasResizeGrip=false;
		DummyStatusbar.Visible=false;
		
		// give resize grip to a statusbar
		if (OverwriteStatusbar.Visible==false){
			if (SelectionStatusbar.Visible==false) {
				if (OffsetStatusbar.Visible==false)
					DummyStatusbar.Visible=true; 
				else
					OffsetStatusbar.HasResizeGrip=true;
			}
			else
				SelectionStatusbar.HasResizeGrip=true;	
		}
		else
			OverwriteStatusbar.HasResizeGrip=true;
	}

}

} // end namespace
